// Save bytes in the minified (but not gzipped) version:
var ArrayProto = Array.prototype,
    ObjProto = Object.prototype,
    FuncProto = Function.prototype,

// Create quick reference variables for speed access to core prototypes.
    push = ArrayProto.push,
    slice = ArrayProto.slice,
    toString = ObjProto.toString,
    hasOwnProperty = ObjProto.hasOwnProperty,

// All **ECMAScript 5** native function implementations that we hope to use are declared here.
    nativeIsArray = Array.isArray,
    nativeKeys = Object.keys,
    nativeBind = FuncProto.bind,
    nativeCreate = Object.create,
    property = function (key) {
        return function (obj) {
            return obj == null ? undefined : obj[key];
        }
    },
    getLength = property('length'),
    isString = (obj)=> typeof obj === 'string',
    isArray = nativeIsArray || ((obj)=> toString.call(obj) === '[object Array]'),
    isArrayLike = function (collection) {
        if (isString(collection)) {
            return false;
        }
        length = getLength(collection);
        return typeof length == 'number' && length >= 0;
    },
    isObject = function (obj) {
        var type = typeof obj;
        return (type === 'function') || (type === 'object' && !!obj);
    },

    /**
     * Flatten a array
     *
     * @param {*[]} input
     * @param {boolean} shallow
     * @param {boolean} strict
     * @param {number=} startIndex
     * @returns {Array}
     */
    flatten = function (input, shallow, strict, startIndex) {
        var output = [],
            idx = 0,
            i = startIndex || 0,
            length = getLength(input);

        while (i < length) {
            let value = input[i];
            if (isArrayLike(value)) {
                // flatten current level of array or arguments object
                if (!shallow) {
                    value = flatten(value, shallow, strict);
                }

                let j = 0,
                    len = value.length;

                output.length += len;

                while (j < len) {
                    output[idx++] = value[j++];
                }
            } else if (!strict) {
                output[idx++] = value;
            }
            i++;
        }

        return output;
    };


class Utility {
    /**
     * Flatten a array
     *
     * @param {*[]} array
     * @param {boolean=} shallow
     * @returns {Array}
     */
    static flatten(array, shallow) {
        return flatten(array, shallow, false);
    }
}

export default Utility;